﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using LinqExtender;

namespace LinqToTwitter.Core
{
    public class TwitterContext : Query<Twit>
    {
        protected override bool AddItem()
        {
            string username = (string)Bucket.Instance.For.Item(QueryItems.USERNAME).Value;
            string password = (string)Bucket.Instance.For.Item(QueryItems.PASSWORD).Value;

            Validate(username, password);

            string statusText = (string)Bucket.Instance.For.Item("Text").Value;
            string source = (string)Bucket.Instance.For.Item("Source").Value;
       
            Twitter twitter = new Twitter();

            if (string.IsNullOrEmpty(source))
            {
                twitter.Source = source;
            }

            string response = twitter.Update(username, password, statusText, Twitter.OutputFormatType.XML);

            Twit twit = XmlToObject<Twit>.Deserialize(response);
            Type twitType = twit.GetType();

            Bucket.Instance.For.EachItem
            .Process(delegate(BucketItem item)
             {
                 item.Value = twitType.GetProperty(item.Name).GetValue(twit, null);
             });

            return true;
        }

        protected override void Process(LinqExtender.Interface.IModify<Twit> items)
        {
            string response = string.Empty;

            Twitter twitter = new Twitter();

            /// general select statement with no where clause.
            if (!Bucket.Instance.IsDirty)
            {
                response = twitter.GetPublicTimeline(Twitter.OutputFormatType.XML);
            }
            else
            {
                    string username = (string)Bucket.Instance.For.Item(QueryItems.USERNAME).Value;
                    string password = (string)Bucket.Instance.For.Item(QueryItems.PASSWORD).Value;
                    object obj = (Timeline)Bucket.Instance.For.Item(QueryItems.TIMELINE).Value;

                    if (obj == null)
                    {
                        throw new Exception("Must provide a valid timeline");
                    }

                    Timeline timeline = (Timeline) obj;

                    if (timeline == Timeline.Friends)
                    {
                        Validate(username, password);
                        response = twitter.GetFriendsTimeline(username, password, Twitter.OutputFormatType.XML);
                    }
                    else if (timeline == Timeline.User)
                    {
                        response = twitter.GetUserTimeline(username, password, Twitter.OutputFormatType.XML);
                    }
                }

                Twits twits = XmlToObject<Twits>.Deserialize(response);

                /// do some extra in-memory stuff which are not supported by API
                IList<Twit> list = GetList(twits);

                items.AddRange(list, true);
        }

        private IList<Twit> GetList(Twits twits)
        {
            IList<Twit> list;
            if (Bucket.Instance.Entity.ItemsToFetch != null)
            {
                list = twits.TwitList.Skip(Bucket.Instance.Entity.ItemsToSkipFromStart)
                    .Take(Bucket.Instance.Entity.ItemsToFetch.Value)
                    .ToList();
            }
            else
            {
                list = twits.TwitList.Skip(Bucket.Instance.Entity.ItemsToSkipFromStart)
                    .ToList();    
            }
            return list;
        }

        protected override bool UpdateItem()
        {
            throw new Exception("Update not supported by twitter.");
        }

        internal class QueryItems
        {
            public const string USERNAME = "Username";
            public const string PASSWORD = "Password";
            public const string TIMELINE = "Timeline";
            public const string SCREEN_NAME = "ScreenName";
        }

        private void Validate(string username, string password)
        {
            if (string.IsNullOrEmpty(username))
            {
                throw  new Exception("Must provide a valid username");
            }

            if (string.IsNullOrEmpty(password))
            {
                throw new Exception("Must provide a valid password");
            }
        }
    }
}
