﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LinqToTwitter.Core;
using NUnit.Framework;
using LinqExtender;

namespace LinqToTwitter.Test
{
    [TestFixture]
    public class BasicTest
    {
        public const string USERNAME = "##USERNAME##";
        public const string PASSWORD = "##PASSWORD##";
        private TwitterContext context;
        
        [SetUp]
        public void Setup()
        {
            context = new TwitterContext();
            context.OnError += new Query<Twit>.ErrorHandler(context_OnError);
        }

        void context_OnError(ProviderException providerException)
        {
            Assert.Fail(providerException.Message);
        }


        [Test]
        public void TestFriendsTimeLine()
        {
            var query = (from twit in context
                         where twit.Username == USERNAME && twit.Password == PASSWORD && twit.Timeline == Timeline.Friends
                         orderby twit.LastUpdated descending
                         select twit);
            /// nothing usefull to do.
            Assert.IsTrue(query.Count() > 0);
        }

        [Test]
        public void TestPublicTimeLine()
        {
            var query =  from twit in context
                         select twit;

            Twit t = query.Last();
            
            Assert.IsTrue(t.User != null);
            /// gets to top 20 twits.
            Assert.AreEqual(20, query.Count());
        }

        [Test]
        public void TestStatusUpdate()
        {
            Twit twitObject = new Twit()
            {
              Username = USERNAME,
              Password = PASSWORD,
              Text = "@work"
            };
            context.Add(twitObject);
            context.SubmitChanges();

            Assert.IsTrue(!string.IsNullOrEmpty(twitObject.Id));
        }

    }
}
